﻿using System;
using System.Globalization;
using System.Linq;

using Platform.Net;

namespace Platform.Geo
{
    public class GeoPolyline : GeoShape
    {
        public GeoPosition[] Points { get; set; }

        /// <summary>
        /// Convert to WKT
        /// </summary>
        /// <returns></returns>
        public override string ToWKT()
        {
            return string.Format("LINESTRING({0})", string.Join(", ", Points.Select(p => string.Format(CultureInfo.InvariantCulture, "{0} {1}", p.Longitude, p.Latitude))));
        }

        /// <summary>
        /// Parse from WKT
        /// </summary>
        /// <param name="wkt"></param>
        /// <returns></returns>
        public static GeoPolyline FromWKT(string wkt)
        {
            if (string.IsNullOrEmpty(wkt)) return null;
            return FromWKT(new NetStringReader(wkt));
        }

        /// <summary>
        /// Parse from WKT
        /// </summary>
        /// <param name="wkt"></param>
        /// <returns></returns>
        public static GeoPolyline FromWKT(NetStringReader reader)
        {
            reader.Skip();
            if (reader.ReadPattern("LINESTRING") == -1) throw new FormatException("Missing literal 'LINESTRING' from wkt");
            var line = new GeoPolyline { Points = ReadWktLineData(reader) };
            return line;
        }

        /// <summary>
        /// Get bounds Of all
        /// </summary>
        /// <returns></returns>
        public override GeoRectangle GetBounds()
        {
            GeoRectangle bounds = null;
            foreach (var point in Points)
                if (bounds == null) bounds = new GeoRectangle { NorthwestCorner = point, SoutheastCorner = point };
                else bounds = bounds.Union(point);
            return bounds;
        }
    }
}
